package framework;

import java.util.Vector;
import javax.microedition.lcdui.Graphics;
import javax.microedition.lcdui.Image;
import no.tellu.cdlc.gui.*;
import no.tellu.commonplatform.serviceframe.common.map.MapBoundingBox;

/**
 * POIElement.java
 * <p/>
 * Draws the points of interest in a map. The POIs can be selected
 * by pressing the number (1-9) they are represented by. When a POI is
 * selected, a popup with some information is drawn. Choosing POIs
 * is done by pressing the fire button.
 * <p/>
 * The POIElement is used together with MapElement and MapLayout in a
 * panel.
 */
public class POIElement extends GuiElement {

	private MapObject currentMap = null;
	
	private Vector allPOIs;
	
	//Vector for pois with popups
	private Vector selectablePOIs;
	
	private Vector notSelectablePOIs;
	
	private Vector savedPOIs;
	
	private int maxNumberOfPOIsToSave;
	
	private MapBoundingBox mbb;
	
	private int oldPOIloc = -1;
	private POIObject selectedPOI = null;
	private int selectedPOIx;
	private int selectedPOIy;
	
	private int savedColour = 0x00FF0000;;

	/**
	 * Constructor POIElement
	 * @param elementId				The id of the object of this class
	 * @param mapHeight				The map height to be set
	 * @param map					The object containing the map information
	 * @param maxNumberOfPOIsToSave	The maximum number of POIS that can be saved
	 */
	public POIElement(String elementId, int mapWidth, int mapHeight, MapObject map,
			int maxNumberOfPOIsToSave) {
		super(elementId);
        setWidth(mapWidth);			
        setHeight(mapHeight);
		setSelectable(true);
		currentMap = map;
		this.maxNumberOfPOIsToSave = maxNumberOfPOIsToSave;
		
		init();
	}
	
	/**
	 * Initialises necessary values.
	 *
	 */
	private void init(){			
		allPOIs = currentMap.getPOIs();
		mbb = currentMap.getMapBoundingBox();	
		savedPOIs = new Vector();
		
		selectablePOIs = new Vector();
		notSelectablePOIs = new Vector();
		
		//Add the selectable and not selectable pois to their vectors
		for(int i = 0; i < allPOIs.size(); i++){			
			POIObject poi = (POIObject)allPOIs.elementAt(i);
			if(poi.isSelectable()) selectablePOIs.addElement(poi);
			else notSelectablePOIs.addElement(poi);
		}
	}
	
	/**
	 * Paints the selectable and not selectable points of interest.
	 */
	public void paint(Graphics g, boolean selected) {		
		if(selectablePOIs.size() != 0) paintPOIs(g, selected, selectablePOIs);		
		if(notSelectablePOIs.size() != 0) paintPOIs(g, selected, notSelectablePOIs);	
	}
	
	/**
	 * Paints points of interest
	 * @param g
	 * @param selected
	 * @param pois			The points of interest
	 */
	private void paintPOIs(Graphics g, boolean selected, Vector pois){	
		GuiFont poiFont = new GuiFont("LARGE");
		
		for(int i = 0; i < pois.size(); i++){
			POIObject poiObject = (POIObject) pois.elementAt(i);

			int x = poiObject.getXCoordinate(getX(), getWidth(), mbb);
			int y = poiObject.getYCoordinate(getY(), getHeight(), mbb);
			
			if(poiObject.getPoiImage() != null){
				//Draw the POI image
				Image poiImage = null;
			    try{
					poiImage = Image.createImage(poiObject.getPoiImage());
					
					g.drawImage(poiImage, x - poiFont.getHeight(), y - poiFont.getHeight(), Graphics.TOP | Graphics.LEFT);
				} catch (Exception e){
				}			
			}
			
            else {
            	//Check if the POI should be marked as saved
            	if(getPOIIndexInSavedVector(poiObject) > -1) g.setColor(savedColour);
            	
            	//Check if the the POI should be marked as selected
            	else if(poiObject.isSelected()) g.setColor(GuiWindow.selectedColour);
    			
            	//The POI should be marked as not selected
            	else g.setColor(GuiWindow.notSelectedColour);
    					
    			//Draw the POI
            	g.fillRoundRect(x - (poiFont.getHeight()/2), y - poiFont.getHeight()/2, poiFont.getHeight(), poiFont.getHeight(), 90, 90);
    			
    			g.setColor(GuiWindow.textFieldTextColour);
    			g.drawRoundRect(x - (poiFont.getHeight()/2), y - poiFont.getHeight()/2, poiFont.getHeight(), poiFont.getHeight(), 90, 90);
    			
            	drawString(g, "" + (i + 1), x - getWidtOfString("" + (i + 1)) / 2, y - poiFont.getHeight() / 2 + 2, Graphics.TOP | Graphics.LEFT);
			}
			
			if(poiObject.isSelected()){
				selectedPOI = poiObject;	
				selectedPOIx = x;
				selectedPOIy = y;
			}
		}	
		if(selectedPOI !=null) paintPopup(g, selectedPOIx, selectedPOIy, selectedPOI);
	}
		
	/**
	 * Paints a popup next to a selected POI, with some information
	 * @param g
	 * @param x
	 * @param y
	 * @param poiObject		The selected POI
	 */
	private void paintPopup(Graphics g, int x, int y, POIObject poiObject) {
		
		GuiFont contentPopupFont = new GuiFont("MEDIUM_PLAIN");		
		GuiFont headingPopupFont = new GuiFont("MEDIUM_BOLD");
			
		//Finds the maximum width of the strings in the popup:
		int lengthOfField = 0;
		lengthOfField = getWidtOfString(poiObject.getInfo().getName(), headingPopupFont);
		lengthOfField += 15;
		int temp = getWidtOfString(poiObject.getInfo().getStreetAddress(), contentPopupFont);
		if(temp > lengthOfField) lengthOfField = temp;
		
		String popupInfo = poiObject.getInfo().getExtraPopupInfo();
		if(popupInfo != null && popupInfo.length() > 0) {
			temp = getWidtOfString(popupInfo);
		
			if(temp > lengthOfField) lengthOfField = temp;
		}

		//Finds the height of the popup
		int heightOfField = 13;
		if(popupInfo != null && popupInfo.length() > 0) heightOfField += 2 * contentPopupFont.getHeight();
		else heightOfField += contentPopupFont.getHeight() + 3;
		
		//Checks if there is enough space for the popup above the POI
		if((y - heightOfField - contentPopupFont.getHeight()/4 - 4) < (getY() + 1)){
			//Draw the popup under the POI:
			y += heightOfField + (contentPopupFont.getHeight()/4)*3 + 3;
		}
		
		//Checks if there is enough space for the popup on the right side of the POI
		if(x + contentPopupFont.getHeight()/4 + lengthOfField > getX() + getWidth()){			
			//Draws the popup on the left side of the POI
			x -= ((contentPopupFont.getHeight()/4)*3 + lengthOfField);			
			//Checks if there is enough space for the popup on the left side of the POI
			if(x < getX() + 1){			
				//Moves the popup towards the right until all of it is shown
				while(x < getX() + 1) x++;
			}
		}

		g.setColor(GuiDisplay.getSkin().backgroundColour);
		
		//Draws the popup with content
		g.fillRect(x + contentPopupFont.getHeight()/4, y - heightOfField - contentPopupFont.getHeight()/4 - 4, lengthOfField + 4, heightOfField);
		
		g.setColor(GuiWindow.headingColour);
		
		g.drawRect(x + contentPopupFont.getHeight()/4, y - heightOfField - contentPopupFont.getHeight()/4 - 4, lengthOfField + 4, heightOfField);
		
		headingPopupFont.drawString(g, (poiObject.getInfo()).getName(), x + contentPopupFont.getHeight()/4 + 3, y - heightOfField - contentPopupFont.getHeight()/4 - 3, Graphics.TOP | Graphics.LEFT);
		
		contentPopupFont.drawString(g, (poiObject.getInfo()).getStreetAddress(), x + 
				contentPopupFont.getHeight()/4 + 3, y - heightOfField - contentPopupFont.getHeight()/4 + 
				contentPopupFont.getHeight() - 6, Graphics.TOP | Graphics.LEFT);	
	
		if(popupInfo != null && popupInfo.length() > 0) contentPopupFont.drawString(g, popupInfo, x + 
				contentPopupFont.getHeight()/4 + 3, y - heightOfField - contentPopupFont.getHeight()/4 + 
				contentPopupFont.getHeight()* 2 - 9, Graphics.TOP | Graphics.LEFT);	
		
		}
		
	/**
	 * Saves a POI in the saved POIs vector 
	 * @return false if the POI already has been saved, true otherwise
	 */
	public boolean savePOI(){
		if(oldPOIloc != -1){							
			POIObject poiObject = (POIObject) selectablePOIs.elementAt(oldPOIloc);
			if(getPOIIndexInSavedVector(poiObject) > -1) return false;							
			if(savedPOIs.size() < maxNumberOfPOIsToSave){
				savedPOIs.addElement(poiObject);
			}
		}
		return true;
	}
	
	/**
	 * Deletes a saved POI
	 */
	public void deleteSavedPOI(){
		if(oldPOIloc != -1){
			if(savedPOIs.size() > 0){
				POIObject poiObject = (POIObject) selectablePOIs.elementAt(oldPOIloc);
				int savedIndex = getPOIIndexInSavedVector(poiObject);				
				if(savedIndex != -1) savedPOIs.removeElementAt(savedIndex);							}
		}
	}
	
	/**
	 * Finds the index number of a POI in the saved POIs vector
	 * @param poiObjectOnMap	the POI to be searched for
	 * @return the index number or -1 if the POI isn't in the vector
	 */
	public int getPOIIndexInSavedVector(POIObject poiObjectOnMap){
		int index = -1;
		for(int i = 0; i < savedPOIs.size(); i++){
			POIObject poiObjectSaved = (POIObject) savedPOIs.elementAt(i);				
			if(poiObjectOnMap.equals(poiObjectSaved)) index = i;			
		}
		return index;	
	}
	
	/**
	 * 
	 * @return the vector with saved points of interest
	 */
	public Vector getSavedPOIs() {
		return savedPOIs;
	}

	/**
	 * Sets the saved POIs.
	 * @param savedPOIs the vector with saved POIs
	 */
	public void setSavedPOIs(Vector savedPOIs) {
		this.savedPOIs = savedPOIs;
	}

	/**
	 * Keeps control of which kindergarten is selected by the user.
	 */
	public boolean numberPressed(int key) {
		if(key <= selectablePOIs.size()){
			
			if(key > 0){	
				if(oldPOIloc != -1) {
					//Set the previous selected POI to 'not selected'
					POIObject oldPOI = (POIObject) selectablePOIs.elementAt(oldPOIloc);					
					oldPOI.setSelected(false);
			
					selectablePOIs.insertElementAt(oldPOI, oldPOIloc);
					selectablePOIs.removeElementAt(oldPOIloc + 1);
				}
				//Fetch the new selected POI and set it to 'selected'
				oldPOIloc = key - 1;
				POIObject poi = (POIObject) selectablePOIs.elementAt(oldPOIloc);
				poi.setSelected(true);
				selectablePOIs.insertElementAt(poi, oldPOIloc);
				selectablePOIs.removeElementAt(oldPOIloc + 1);
				
				selectedPOI = poi;
			}					
			return true;
		}
		return false;
	}
	
	/**
	 * Saves the selected POI if it is not saved, deletes it otherwise.
	 */
	public boolean fire(int arg0) {
		if(!savePOI()) deleteSavedPOI();
		return true;
	}

	/**
	 * 
	 * @return the highest number of kindergartens possible to save.
	 */
	public int getMaxNumberOfPOIsToSave() {
		return maxNumberOfPOIsToSave;
	}

	/**
	 * Sets the highest number of kindergartens possible to save.
	 * @param maxNumberOfPOIsToChoose the highest number of kindergartens possible to save
	 */
	public void setMaxNumberOfPOIsToSave(int maxNumberOfPOIsToChoose) {
		this.maxNumberOfPOIsToSave = maxNumberOfPOIsToChoose;
	}

	/**
	 * 
	 * @return the selected point of interest
	 */
	public POIObject getSelectedPOI() {
		return selectedPOI;
	}

	/**
	 * Sets the selected point of interest.
	 * @param selectedPOI the point of interest to be set as selected
	 */
	public void setSelectedPOI(POIObject selectedPOI) {
		this.selectedPOI = selectedPOI;
	}
	
}
